import httpx
from typing import Union
import re, mwparserfromhell


class arktool:
    def __init__(
        self,
        domains: list = [
            "https://raw.githubusercontent.com/",
            "https://raw.kgithub.com/",
            "https://ghproxy.com/https://raw.githubusercontent.com/",
            "https://fastly.jsdelivr.net/gh/",
            "https://cdn.staticaly.com/gh/",
            "https://ghproxy.net/https://raw.githubusercontent.com/",
            "https://gcore.jsdelivr.net/gh/",
            "https://jsdelivr.b-cdn.net/gh/",
        ],
    ):
        """同步构造函数"""
        self.domains = domains
        timeout = httpx.Timeout(10.0, connect=60.0, read=60.0, write=60.0, pool=60.0)
        self.client = httpx.AsyncClient(verify=False, timeout=timeout)
        self.headers = {"User-Agent": f"mwbot/arktool"}

    async def __aexit__(self):
        """异步析构函数"""
        await self.client.aclose()

    async def read_ark_text(self, filename: str) -> str:
        for i in self.domains:
            try:
                res = await self.client.get(
                    url=f"{i}/Kengxxiao/ArknightsGameData/master/zh_CN/gamedata/{filename}",
                    headers=self.headers,
                )
                return res.text
            except:
                pass
        return None

    async def read_ark_file(self, filename: str) -> dict:
        for i in self.domains:
            try:
                res = await self.client.get(
                    url=f"{i}/Kengxxiao/ArknightsGameData/master/zh_CN/gamedata/{filename}",
                    headers=self.headers,
                )
                return res.json()
            except:
                pass
        return None

    async def get_item_name(self, id: Union[str, int]) -> str:
        """输出item(物品)ID对应的名称
        .. code-block:: python
            item = get_item_name("30012")
        :param id: 物品ID
        :returns: 对应物品的名称
        """
        res = await self.read_ark_file("excel/item_table.json")
        return res["items"][str(id)]["name"]

    async def deal_item_info(self, type: str, id: str, droptype: Union[str, int]) -> str:
        res = await self.read_ark_file("excel/item_table.json")
        link = res["items"][id]
        name = link["name"]
        if droptype == 8 or droptype == "COMPLETE":
            kind = "三星获得"
        return f"{name}:{kind}"

    async def catch_item_template(self, id: str, count: int) -> str:
        res = await self.read_ark_file("excel/item_table.json")
        return "{{材料消耗|" + str(res["items"][id]["name"]) + "|" + str(count) + "}}"

    def get_stage_id(self, content: str) -> str:
        wikicode = mwparserfromhell.parse(content)
        templates = wikicode.filter_templates()
        for i in templates:
            if i.name.matches("普通关卡信息") or i.name.matches("剿灭关卡信息"):
                return str(i.get("关卡id").value).strip()
        return None

    async def get_stage_info(self, content: str):
        stage_id: str = self.get_stage_id(content=content)
        stage_location: str = ""
        # 处理关卡id对应的文件路径
        if stage_id == "":
            return None
        elif match := re.match(r"^ro(\d+)_", stage_id):
            number = int(match.group(1))
            file = await self.read_ark_file("excel/roguelike_topic_table.json")
            stage_location = file["details"][f"rogue_{number}"]["stages"][stage_id]["levelId"]
        elif stage_id.startswith("mem_"):
            file = await self.read_ark_file("excel/handbook_info_table.json")
            for k, v in file["handbookStageData"].items():
                if v["stageId"] == stage_id:
                    stage_location = v["levelId"]
                    break
        elif stage_id.startswith("tower_"):
            file = await self.read_ark_file("excel/climb_tower_table.json")
            stage_location = file["levels"][stage_id]["levelId"]
        elif stage_id.startswith("act42d0_"):
            file = await self.read_ark_file("excel/activity_table.json")
            stage_location = file["activity"]["TYPE_ACT42D0"]["act42d0"]["stageInfoData"][stage_id]["levelId"]
        else:
            file = await self.read_ark_file("excel/stage_table.json")
            stage_location = file["stages"][stage_id]["levelId"]
        # 返回文件
        if stage_location:
            return await self.read_ark_file(f"levels/{stage_location.lower()}.json")
        else:
            return ""

    async def get_char_name(self, id: str) -> str:
        character_table = await self.read_ark_file("excel/character_table.json")
        return character_table[id]["name"]



"""定义一些功能特化的基类"""
from abc import ABC
import mwparserfromhell

# import pydantic
# import pendulum
# import regex as re


class WikiSectionList(ABC):
    """用于Bot.get_sections()方法的字典类
    覆写了dict.index()方法，输出可直接用于bot.edit(section)的值"""

    def __init__(self, wl: list):
        self.wl = wl

    def index(self, name: str) -> int:
        return self.wl.index(name) + 1

    def __str__(self):
        return str(self.wl)

    @property
    def list(self):
        return self.wl


class gen_wikitext:
    """生成wikitext"""

    def __init__(self):
        self.txt: str = """"""

    def add_title(self, times: int, title: str):
        self.txt += "=" * times + title + "=" * times + "\n"

    def add_text(self, text: str):
        self.txt += text + "\n"

    def add_cat(self, cat: str):
        self.txt += f"[[分类:{cat}]]\n"


class gen_tem:
    """生成wikitext-template"""

    def __init__(self, title: str, with_enter: bool = True):
        self.params = []
        self.with_enter = with_enter
        if self.with_enter:
            self.txt += "\n"

    def add_par(self, name: str = "", txt: str = ""):
        """添加一行参数"""
        if name:
            self.params.append(f"|{name}={txt}")
        else:
            self.params.append(f"|{txt}")

    def result(self) -> str:
        """输出结果"""
        enter_key = ""
        if self.with_enter:
            enter_key = "\n"
        return "{{" + enter_key.join(self.params) + "}}"



"""存储与PRTS编辑相关的方法"""
import mwparserfromhell
import ujson as json
from typing import Union
import re

# from loguru import logger
# import os
# from pathlib import Path


GameDataPosition = "/home/bot/ArknightsGameData/zh_CN/gamedata"

def read_ark_text(filename: str) -> str:
    with open(f"{GameDataPosition}/{filename}", "r", encoding="utf-8") as f:
        content = f.read()
    return content

def read_ark_file(filename: str) -> dict:
    with open(f"{GameDataPosition}/{filename}", "r", encoding="utf-8") as f:
        content = f.read()
    return json.loads(content)


def get_item_name(id: Union[str, int]) -> str:
    """输出item(物品)ID对应的名称
    .. code-block:: python
        item = get_item_name("30012")
    :param id: 物品ID
    :returns: 对应物品的名称
    """
    return read_ark_file("excel/item_table.json")["items"][str(id)]["name"]


def deal_item_info(type: str, id: str, droptype: Union[str, int]) -> str:
    link = read_ark_file("excel/item_table.json")["items"][id]
    name = link["name"]
    if droptype == 8 or droptype == "COMPLETE":
        kind = "三星获得"
    return f"{name}:{kind}"


def catch_item_template(id: str, count: int) -> str:
    return (
        "{{材料消耗|"
        + str(read_ark_file("excel/item_table.json")["items"][id]["name"])
        + "|"
        + str(count)
        + "}}"
    )


def get_stage_id(content) -> str:
    wikicode = mwparserfromhell.parse(content)
    templates = wikicode.filter_templates()
    for i in templates:
        if i.name.matches("普通关卡信息") or i.name.matches("剿灭关卡信息"):
            return str(i.get("关卡id").value).strip()
    return None


def get_stage_info(content: str):
    stage_id: str = get_stage_id(content=content)
    # 处理关卡id对应的文件路径
    if stage_id == "":
        return None
    elif match := re.match(r"^ro(\d+)_", stage_id):
        number = int(match.group(1))
        stage_location = read_ark_file("excel/roguelike_topic_table.json")["details"][f"rogue_{number}"]["stages"][stage_id]["levelId"]
    elif stage_id.startswith("mem_"):
        for k, v in read_ark_file("excel/handbook_info_table.json")["handbookStageData"].items():
            if v["stageId"] == stage_id:
                stage_location = v["levelId"]
                break
    elif stage_id.startswith("tower_"):
        stage_location = read_ark_file("excel/climb_tower_table.json")["levels"][stage_id]["levelId"]
    elif stage_id.startswith("act42d0_"):
        stage_location = read_ark_file("excel/activity_table.json")["activity"]["TYPE_ACT42D0"]["act42d0"]["stageInfoData"][stage_id]["levelId"]
    else:
        stage_location = read_ark_file("excel/stage_table.json")["stages"][stage_id]["levelId"]
    # 返回文件
    if stage_location:
        return read_ark_file(f"levels/{stage_location.lower()}.json")
    else:
        return ""


class char:
    def __init__(self) -> None:
        self.character_table = read_ark_file("excel/character_table.json")

    def get_char_name(self, id: str) -> str:
        return self.character_table[id]["name"]
